package com.xs.core.service.Impl.team;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.ObjUtil;
import cn.hutool.core.util.StrUtil;
import com.xs.core.model.coin.msg.TeamShareMessage;
import com.xs.core.model.team.entity.TeamInviteConfig;
import com.xs.core.model.team.entity.TeamInviteRewardRecord;
import com.xs.core.model.team.entity.TeamShareRewardRecord;
import com.xs.core.model.team.resp.TeamUserResp;
import com.xs.core.model.team.resp.TeamsInfoResp;
import com.xs.core.model.user.entity.TgUser;
import com.xs.core.service.team.ITeamInviteConfigService;
import com.xs.core.service.team.ITeamInviteRewardRecordService;
import com.xs.core.service.team.ITeamShareRewardRecordService;
import com.xs.core.service.team.TeamShareService;
import com.xs.core.service.user.ITgUserService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;


@Service
@Slf4j
public class TeamShareGoldCoinSettlementServiceImpl implements TeamShareService {
    @Autowired
    private ITgUserService userService;

    @Autowired
    private ITeamInviteConfigService teamInviteConfigService;

    @Autowired
    private ITeamInviteRewardRecordService teamInviteRewardRecordService;

    @Autowired
    private ITeamShareRewardRecordService teamShareRewardRecordService;

    @Override
    @Transactional
    public void teamShareGoldCoinSettlement(TeamShareMessage teamShareMessage) {
        Long userId = teamShareMessage.getUserId();
        //获取用户的上一级邀请人 即a队
        TgUser inviteA = userService.getInviterByUserId(userId);
        TgUser inviteB = null;
        if (inviteA != null && StrUtil.isNotBlank(inviteA.getReferrerId())) {
            //获取a队的上一级邀请人 即b队
            inviteB = userService.getById(inviteA.getReferrerId());
        }
        TeamInviteConfig config = teamInviteConfigService.getTeamInviteConfig();
        List<TeamShareRewardRecord> saveList = new ArrayList<>();
        if (ObjUtil.isNotNull(inviteA)) {
            //发放a队金币
            TeamShareRewardRecord teamA = new TeamShareRewardRecord();
            teamA.setTargetId(inviteA.getId());
            BigDecimal yieldTotal = teamShareMessage.getGoldCoinYieldTotal();
            //计算a队的奖励
            BigDecimal multiply = yieldTotal.multiply(new BigDecimal(config.getTeamAScale()));
            teamA.setRewardAmount(multiply.setScale(2, RoundingMode.HALF_UP));
            teamA.setTeamType("A");
            teamA.setGoldCoinYieldTotal(yieldTotal);
            teamA.setCreateTime(LocalDateTime.now());
            teamA.setScale(config.getTeamAScale());
            teamA.setInviteeId(userId);
            teamA.setBatchId(teamShareMessage.getBatchId());
            teamA.setClaimStatus(0);
            saveList.add(teamA);
        }
        if (ObjUtil.isNotNull(inviteB)) {
            //发放b队金币
            TeamShareRewardRecord teamB = new TeamShareRewardRecord();
            teamB.setTargetId(inviteB.getId());
            BigDecimal yieldTotal = teamShareMessage.getGoldCoinYieldTotal();
            BigDecimal multiply = yieldTotal.multiply(new BigDecimal(config.getTeamBScale()));
            teamB.setRewardAmount(multiply.setScale(2, RoundingMode.HALF_UP));
            teamB.setTeamType("B");
            teamB.setGoldCoinYieldTotal(yieldTotal);
            teamB.setCreateTime(LocalDateTime.now());
            teamB.setScale(config.getTeamBScale());
            teamB.setInviteeId(userId);
            teamB.setBatchId(teamShareMessage.getBatchId());
            teamB.setClaimStatus(0);
            saveList.add(teamB);
        }
        teamShareRewardRecordService.saveBatch(saveList);
    }

    @Override
    public void teamInviteGoldCoinRewardGeneral(Long inviterId, Long inviteeId) {
        TeamInviteRewardRecord record = new TeamInviteRewardRecord();
        record.setReceiverId(inviterId);
        record.setInviteeId(inviteeId);
        record.setRewardAmount(new BigDecimal(1000));
        record.setCreateTime(LocalDateTime.now());
        record.setClaimStatus(0);
        teamInviteRewardRecordService.save(record);
    }

    @Override
    public TeamsInfoResp getMyTeamsInfo(Long id) {
        List<TeamUserResp> teamUserList = getMyTeamUserList(id, null);
        TeamsInfoResp teamsInfoResp = new TeamsInfoResp();
        TeamInviteConfig config = teamInviteConfigService.getTeamInviteConfig();
        if (CollUtil.isNotEmpty(teamUserList)) {
            teamsInfoResp.setTotal(teamUserList.size());
            long countA = teamUserList.stream().filter(t -> "A".equals(t.getTeamType())).count();
            long countB = teamUserList.stream().filter(t -> "B".equals(t.getTeamType())).count();
            teamsInfoResp.setTeamACount(countA);
            teamsInfoResp.setTeamBCount(countB);
        } else {
            teamsInfoResp.setTotal(0);
            teamsInfoResp.setTeamACount(0L);
            teamsInfoResp.setTeamBCount(0L);
        }
        if (ObjUtil.isNotNull(config)) {
            teamsInfoResp.setProportionA(config.getTeamAScale());
            teamsInfoResp.setProportionB(config.getTeamBScale());
        }
        return teamsInfoResp;
    }

    @Override
    public List<TeamUserResp> getMyTeamUserList(Long userId, String teamType) {
        return userService.getMyTeamUserList(userId, teamType);
    }
}
